<?php

namespace Drupal\gcommerce\EventSubscriber;

use Drupal\group\GroupMembershipLoaderInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\commerce_cart\Event\CartEntityAddEvent;
use Drupal\commerce_cart\Event\CartEvents;

/**
 * Add to cart EventSubscriber for groups.
 */
class GroupCartEventSubscriber implements EventSubscriberInterface {

  /**
   * The membership loader service.
   *
   * @var \Drupal\group\GroupMembershipLoaderInterface
   */
  protected $membershipLoader;

  /**
   * Constructs a GroupCartEventSubscriber object.
   *
   * @param \Drupal\group\GroupMembershipLoaderInterface $membership_loader
   *   The group membership loader service.
   */
  public function __construct(GroupMembershipLoaderInterface $membership_loader) {
    $this->membershipLoader = $membership_loader;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [CartEvents::CART_ENTITY_ADD => 'onCartEntityAdd'];
  }

  /**
   * Add new cart entity to group content for user groups.
   *
   * @param \Drupal\commerce_cart\Event\CartEntityAddEvent $event
   *   The add to cart event.
   */
  public function onCartEntityAdd(CartEntityAddEvent $event) {
    $cart = $event->getCart();
    $user = $cart->getCustomer();
    $user_groups = $this->membershipLoader->loadByUser($user);

    // @todo: Add functionality that will load allowed groups.
    // Allowed groups for which you can add a cart entity.
    $allowed_group_type_ids = ['company'];
    foreach ($user_groups as $membership) {
      $group = $membership->getGroup();
      $group_type = $group->getGroupType();
      if (!in_array($group_type->id(), $allowed_group_type_ids)) {
        continue;
      }

      // Add the entity of the cart to the groups, if it has not been added yet.
      $content_plugin_id = 'group_order:' . $cart->bundle();
      if (
        $group_type->hasContentPlugin($content_plugin_id) &&
        empty($group->getContentByEntityId($content_plugin_id, $cart->id()))
      ) {
        $group->addContent($cart, $content_plugin_id);
      }
    }
  }

}
